==> pueue-1.0.0-rc.4.crate: pueue-1.0.0-rc.4/tests/unix/group.rs <==
use anyhow::Result;

use pueue_lib::network::message::*;

use crate::helper::*;

#[tokio::test(flavor = "multi_thread", worker_threads = 2)]
/// Add and directly remove a group.
async fn test_add_and_remove() -> Result<()> {
    let (settings, tempdir) = base_setup()?;
    let shared = &settings.shared;
    let _pid = boot_daemon(tempdir.path())?;

    // Add a new group
    let add_message = Message::Group(GroupMessage::Add("testgroup".to_string()));
    assert_success(send_message(shared, add_message.clone()).await?);
    wait_for_group(shared, "testgroup").await?;

    // Make sure it got added
    let state = get_state(shared).await?;
    assert!(state.groups.contains_key("testgroup"));

    // Try to add the same group again. This should fail
    assert_failure(send_message(shared, add_message).await?);

    // Remove the newly added group
    let remove_message = Message::Group(GroupMessage::Remove("testgroup".to_string()));
    assert_success(send_message(shared, remove_message.clone()).await?);

    // Make sure it got removed
    let state = get_state(shared).await?;
    assert!(!state.groups.contains_key("testgroup"));

    Ok(())
}

#[tokio::test(flavor = "multi_thread", worker_threads = 2)]
/// Users cannot delete the default group.
async fn test_cannot_delete_default() -> Result<()> {
    let (settings, tempdir) = base_setup()?;
    let shared = &settings.shared;
    let _pid = boot_daemon(tempdir.path())?;

    let pause_message = Message::Group(GroupMessage::Remove("default".to_string()));
    assert_failure(send_message(shared, pause_message).await?);

    Ok(())
}

#[tokio::test(flavor = "multi_thread", worker_threads = 2)]
/// Users cannot delete a non-existing group.
async fn test_cannot_delete_non_existing() -> Result<()> {
    let (settings, tempdir) = base_setup()?;
    let shared = &settings.shared;
    let _pid = boot_daemon(tempdir.path())?;

    let pause_message = Message::Group(GroupMessage::Remove("doesnt_exist".to_string()));
    assert_failure(send_message(shared, pause_message).await?);

    Ok(())
}

#[tokio::test(flavor = "multi_thread", worker_threads = 2)]
/// Groups with tasks shouldn't be able to be removed.
async fn test_cannot_delete_group_with_tasks() -> Result<()> {
    let (settings, tempdir) = base_setup()?;
    let shared = &settings.shared;
    let _pid = boot_daemon(tempdir.path())?;

    // Add a new group
    let add_message = Message::Group(GroupMessage::Add("testgroup".to_string()));
    assert_success(send_message(shared, add_message.clone()).await?);
    wait_for_group(shared, "testgroup").await?;

    // Add a task
    assert_success(fixtures::add_task_to_group(shared, "ls", "testgroup").await?);
    wait_for_task_condition(&settings.shared, 0, |task| task.is_done()).await?;

    // We shouldn't be capable of removing that group
    let pause_message = Message::Group(GroupMessage::Remove("testgroup".to_string()));
    assert_failure(send_message(shared, pause_message).await?);

    // Remove the task from the group
    let remove_message = Message::Remove(vec![0]);
    send_message(shared, remove_message).await?;

    // Removal should now work.
    let pause_message = Message::Group(GroupMessage::Remove("testgroup".to_string()));
    assert_success(send_message(shared, pause_message).await?);

    Ok(())
}

==> pueue-1.0.2.crate: pueue-1.0.2/tests/unix/group.rs <==
use anyhow::Result;

use pueue_lib::network::message::*;

use crate::helper::*;

#[tokio::test(flavor = "multi_thread", worker_threads = 2)]
/// Add and directly remove a group.
async fn test_add_and_remove() -> Result<()> {
    let (settings, tempdir) = base_setup()?;
    let shared = &settings.shared;
    let _pid = boot_daemon(tempdir.path())?;

    // Add a new group
    add_group_with_slots(shared, "testgroup", 1).await?;

    // Try to add the same group again. This should fail
    let add_message = Message::Group(GroupMessage::Add("testgroup".to_string()));
    assert_failure(send_message(shared, add_message).await?);

    // Remove the newly added group and wait for the deletion to be processed.
    let remove_message = Message::Group(GroupMessage::Remove("testgroup".to_string()));
    assert_success(send_message(shared, remove_message.clone()).await?);
    wait_for_group_absence(shared, "testgroup").await?;

    // Make sure it got removed
    let state = get_state(shared).await?;
    assert!(!state.groups.contains_key("testgroup"));

    Ok(())
}

#[tokio::test(flavor = "multi_thread", worker_threads = 2)]
/// Users cannot delete the default group.
async fn test_cannot_delete_default() -> Result<()> {
    let (settings, tempdir) = base_setup()?;
    let shared = &settings.shared;
    let _pid = boot_daemon(tempdir.path())?;

    let pause_message = Message::Group(GroupMessage::Remove("default".to_string()));
    assert_failure(send_message(shared, pause_message).await?);

    Ok(())
}

#[tokio::test(flavor = "multi_thread", worker_threads = 2)]
/// Users cannot delete a non-existing group.
async fn test_cannot_delete_non_existing() -> Result<()> {
    let (settings, tempdir) = base_setup()?;
    let shared = &settings.shared;
    let _pid = boot_daemon(tempdir.path())?;

    let pause_message = Message::Group(GroupMessage::Remove("doesnt_exist".to_string()));
    assert_failure(send_message(shared, pause_message).await?);

    Ok(())
}

#[tokio::test(flavor = "multi_thread", worker_threads = 2)]
/// Groups with tasks shouldn't be able to be removed.
async fn test_cannot_delete_group_with_tasks() -> Result<()> {
    let (settings, tempdir) = base_setup()?;
    let shared = &settings.shared;
    let _pid = boot_daemon(tempdir.path())?;

    // Add a new group
    add_group_with_slots(shared, "testgroup", 1).await?;

    // Add a task
    assert_success(fixtures::add_task_to_group(shared, "ls", "testgroup").await?);
    wait_for_task_condition(&settings.shared, 0, |task| task.is_done()).await?;

    // We shouldn't be capable of removing that group
    let pause_message = Message::Group(GroupMessage::Remove("testgroup".to_string()));
    assert_failure(send_message(shared, pause_message).await?);

    // Remove the task from the group
    let remove_message = Message::Remove(vec![0]);
    send_message(shared, remove_message).await?;

    // Removal should now work.
    let pause_message = Message::Group(GroupMessage::Remove("testgroup".to_string()));
    assert_success(send_message(shared, pause_message).await?);

    Ok(())
}
